# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

import bpy
import os
from bpy.props import StringProperty

bl_info = {
    "name": "Import Bezier Patches",
    "author": "Kevin Mather",
    "version": (0, 0, 1),
    "blender": (2, 7, 8),
    "location": "File > Import > Bezier Patches",
    "description": "Import Bezier patches from text file",
    "wiki_url": "no-wiki",
    "category": "Import-Export",
}

__version__ = '.'.join([str(s) for s in bl_info['version']])

upscale = 5.0

def text_line_value(rawtext) :
    retval = -1;
    trimmed = rawtext.rstrip()
    if trimmed.isnumeric() :
        retval = int(trimmed)
    return retval

def construct_vertex(rawtext) :
    global upscale
    trimmed = rawtext.rstrip()
    values = trimmed.split(',')
    retval = (float(values[0]) * upscale, float(values[1]) * upscale, float(values[2]) * upscale)
    return retval
        
class IMPORT_OT_bezier(bpy.types.Operator):
    """Import Bezier patches from file"""
    bl_idname = "import_scene.bez"
    bl_description = 'Import Bezier patches from file'
    #bl_label = "Import Bezier v." + __version__
    bl_label = "Import Bezier"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_options = {'UNDO'}

    filepath = StringProperty(
            name="input file",
            subtype='FILE_PATH'
            )

    def make_empties(self) : # TEST!
        seq = 1
        for onevert in self.verts :
            namnum = '{0}'.format(seq)
            vname = 'empty.' + namnum.zfill(3)
            # make an empty
            emp_ob = bpy.data.objects.new(vname, None)
            emp_ob.location = onevert
            emp_ob.empty_draw_size = 0.02
            emp_ob.show_name = True
            # add to scene
            bpy.context.scene.objects.link(emp_ob)
            seq = seq + 1

    def patches_to_faces(self) :
        # each group of 4 represents one face in patch (positions in array)
        faceorder = [1, 2, 6, 5,
                     2, 3, 7, 6,
                     3, 4, 8, 7,
                     5, 6, 10, 9,
                     6, 7, 11, 10,
                     7, 8, 12, 11,
                     9, 10, 14, 13,
                     10, 11, 15, 14,
                     11, 12, 16, 15]
        retval = []
        for onepatch in self.patches :
            j = 0
            for f in range(0, 9) :
                pos1 = faceorder[j+0] - 1
                v1 = onepatch[pos1]
                pos2 = faceorder[j+1] - 1
                v2 = onepatch[pos2]
                pos3 = faceorder[j+2] - 1
                v3 = onepatch[pos3]
                pos4 = faceorder[j+3] - 1
                v4 = onepatch[pos4]
                fa = (v1, v2, v3, v4)
                retval.append(fa)
                j = j + 4
        return retval

    def do_output(self, meshname) :
        m = bpy.data.meshes.new(meshname)
        noedges = []
        faces = self.patches_to_faces()
        m.from_pydata(self.verts, noedges, faces);
        # add to scene
        mo = bpy.data.objects.new(meshname, m)
        bpy.context.scene.objects.link(mo)

    def parse_patch(self, rawtext) :
        trimmed = rawtext.rstrip()
        valuelist = trimmed.split(',')
        retval = []
        for oneval in valuelist :
            vtxnum = int(oneval) - 1
            retval.append(vtxnum)
        return retval

    def execute(self, context):
        #scene = bpy.data.scenes.new(os.path.basename(self.filepath).replace(".dxf", ""))
        fullpath = os.path.abspath(self.filepath)
        basename = os.path.basename(self.filepath)
        print(fullpath) # TEST!
        f = open(fullpath, 'r')
        textlines = f.readlines()
        f.close()
        linecnt = len(textlines)
        print(linecnt, ' line(s)') # TEST!
        linepos = 0
        num_patch = text_line_value(textlines[linepos])
        linepos = linepos + 1
        print(num_patch, 'patch(es)') # TEST!
        self.patches = []
        for j in range(0, num_patch) :
            pt = self.parse_patch(textlines[linepos])
            self.patches.append(pt)
            linepos = linepos + 1
        num_verts = text_line_value(textlines[linepos])
        linepos = linepos + 1
        print(num_verts, 'vert(s)') # TEST!
        self.verts = []
        for j in range(0, num_verts) :
            vtx = construct_vertex(textlines[linepos])
            self.verts.append(vtx)
            linepos = linepos + 1
        #self.make_empties() # TEST!
        self.do_output(basename)
        return {'FINISHED'}

    def invoke(self, context, event):
        wm = context.window_manager
        wm.fileselect_add(self)
        return {'RUNNING_MODAL'}


def menu_func(self, context):
    self.layout.operator(IMPORT_OT_bezier.bl_idname, text="Bezier patches")


def register():
    bpy.utils.register_module(__name__)
    bpy.types.INFO_MT_file_import.append(menu_func)


def unregister():
    bpy.utils.unregister_module(__name__)
    bpy.types.INFO_MT_file_import.remove(menu_func)

if __name__ == "__main__":
    register()
